package com.lqkj.link.module.zone.service;

import com.lqkj.link.module.authority.domain.UserInfo;
import com.lqkj.link.module.authority.repository.UserInfoRepository;
import com.lqkj.link.module.zone.domain.ModelCategory;
import com.lqkj.link.module.zone.domain.ModelInfo;
import com.lqkj.link.module.zone.repository.ModelCategoryRepository;
import com.lqkj.link.module.zone.repository.ModelInfoRepository;
import com.lqkj.link.util.AliOSSUtils;
import com.lqkj.link.util.FileUtils;
import com.lqkj.link.util.Unzipper;
import org.apache.commons.compress.archivers.ArchiveException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ResourceService {
    private final ModelCategoryRepository categoryRepository;
    private final ModelInfoRepository infoRepository;
    private final UserInfoRepository userInfoRepository;

    @Autowired
    private AliOSSUtils aliOSSUtils;

    public ResourceService(ModelCategoryRepository categoryRepository, ModelInfoRepository infoRepository, UserInfoRepository userInfoRepository) {
        this.categoryRepository = categoryRepository;
        this.infoRepository = infoRepository;
        this.userInfoRepository = userInfoRepository;
    }

    public Page<ModelCategory> pageQuery(String categoryName, Integer page, Integer pageSize) {
        Pageable pageable = PageRequest.of(page, pageSize);
        Page<ModelCategory> categoryPage = categoryRepository.pageQuery(categoryName, pageable);
        for (ModelCategory category : categoryPage.getContent()) {
            category.setModelCount(infoRepository.countWithCategory(category.getCategoryId()));
        }
        return categoryPage;
    }

    public ModelCategory detail(Integer categoryId) {
        //        modelCategory.setModelInfoList(infoRepository.listWithCategory(categoryId));
        return categoryRepository.findById(categoryId).get();
    }

    @Transactional
    public String save(ModelCategory category) throws IOException, ArchiveException {
        if ((category.getCategoryId() != null && categoryRepository.hasSameNameWithoutOne(category.getCategoryName(), category.getCategoryId()))
            || (category.getCategoryId() == null && categoryRepository.hasSameName(category.getCategoryName()))) {
            return "资源名称不能重复!";
        }
        Boolean uploadCompress = category.getUploadCompress();
        category = categoryRepository.save(category);
        String filePath = category.getCompressFilePath();
        if (uploadCompress) {
//            infoRepository.deleteWithCategory(category.getCategoryId());
//            Unzipper.unzip("." + filePath, "./upload/resource/" + category.getCategoryId() + "/", false);
            Unzipper.unZipFiles(new File("." + filePath), "./upload/resource/" + category.getCategoryId() + "/");

            Map<String, ModelInfo> modelInfoMap = infoRepository
                    .listWithCategory(category.getCategoryId())
                    .stream()
                    .collect(Collectors.toMap(ModelInfo::getModelName, ModelInfo -> ModelInfo));
            String modelFolderPath = "./upload/resource/" + category.getCategoryId() + "/model";
            File modelFolder = new File(modelFolderPath);
            if (modelFolder.isDirectory()) {
                List<ModelInfo> list = new ArrayList<>();
                File[] models = modelFolder.listFiles();
                assert models != null;
                for (File model : models) {
                    if (model.isFile() && model.getName().endsWith(".fbx")) {
                        String modelFileName = model.getName();
                        String modelPath = aliOSSUtils.upload(model);
                        // 如果包含中文，重命名
                        String newFileName = UUID.randomUUID() + ".png";
                        String modelIconPath = "." + "/upload/resource/" + category.getCategoryId() + "/icon/" + newFileName;
                        // 模型图标也许要重命名
                        File oldIconFile = new File("." + "/upload/resource/" + category.getCategoryId() + "/icon/" + modelFileName.replace(".fbx", ".png"));
                        if (oldIconFile.exists()) {
                            File newIconFile = new File(modelIconPath);
                            Boolean rename = oldIconFile.renameTo(newIconFile);
                            System.out.println(rename);
                        }
                        if (!modelInfoMap.containsKey(modelFileName.substring(0, modelFileName.lastIndexOf(".")))) {
                            list.add(new ModelInfo(null, category.getCategoryId(), modelFileName.substring(0, modelFileName.lastIndexOf(".")),
                                    null, null, null, modelPath,
                                    modelIconPath.substring(1), null, null));
                        } else {
                            ModelInfo modelInfo = modelInfoMap.get(modelFileName.substring(0, modelFileName.lastIndexOf(".")));
                            modelInfo.setOriginalPath(modelPath);
                            modelInfo.setModelIcon(modelIconPath.substring(1));
                            modelInfo.setJsonPath(null);
                            modelInfo.setTexturePath(null);
                            list.add(modelInfo);
                            modelInfoMap.remove(modelFileName.substring(0, modelFileName.lastIndexOf(".")));
                        }
                    }
                }
                infoRepository.saveAll(list);


                List<Integer> deleteList = modelInfoMap
                        .values()
                        .stream()
                        .map(ModelInfo::getModelId)
                        .toList();

                infoRepository.deleteAllByIdInBatch(deleteList);
            }
        }
        // 更新用户资源刷新状态
        userInfoRepository.updateRefreshStatus();
        return null;
    }

    public void deleteModel(Integer modelId) {
        infoRepository.deleteById(modelId);
    }

    @Transactional
    public void deleteCategory(List<Integer> categoryId) {
        categoryRepository.deleteAllByIdInBatch(categoryId);
        // 更新用户资源刷新状态
        userInfoRepository.updateRefreshStatus();
    }

    public List<Map<String, Object>> resourceCategory(String userCode) {
        return categoryRepository.queryWithUserCode(userCode);
    }

    public List<ModelInfo> models(String userCode) {
        return infoRepository.listWitUserCode(userCode);
    }

    public Map<String, Object> savePersonal(ModelCategory modelCategory, String userCode) {
        Map<String, Object> result = new HashMap<>();
        if (modelCategory.getCategoryId() == 0) {
            modelCategory.setCategoryId(null);
        }
        Integer userId = userInfoRepository.findByUserCode(userCode).getUserId();
        if ((modelCategory.getCategoryId() != null && categoryRepository.hasSameNameByUserWithoutOne(modelCategory.getCategoryName(), modelCategory.getCategoryId(), userId))
                || (modelCategory.getCategoryId() == null && categoryRepository.hasSameNameByUser(modelCategory.getCategoryName(), userId))) {
            result.put("msg", "不能与已有文件夹重名！");
            return result;
        }
        modelCategory.setUserId(userId);
        modelCategory = categoryRepository.save(modelCategory);
        result.put("category", modelCategory);
        return result;
    }

    public ModelInfo saveModel(ModelInfo modelInfo) {
        return infoRepository.save(modelInfo);
    }

    public Boolean needRefreshResource(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        return userInfo.getRefreshResource() != null && userInfo.getRefreshResource();
    }

    public String checkCapacity(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        String filePath = "./upload/resource/model/" + userInfo.getUserId();
        File file = new File(filePath);
        if (!file.exists()) return "0";
        return "个人库 已使用" + FileUtils.convertBytes(FileUtils.getFolderSize(new File(filePath))) + "/20G";
    }

    public String ossCheckCapacity(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        Long aLong = aliOSSUtils.ossCheckCapacity("resource/model/" + userInfo.getUserId() + "/");
        return "个人库 已使用" + FileUtils.convertBytes(aLong) + "/20G";
    }
}
